@@ -1,11 +1,11 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-2000 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2007 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: CatJob.cc,v 1.26 2005/11/29 07:07:55 lav Exp $ */
+/* $Id: CatJob.cc,v 1.31 2008/11/27 05:56:07 lav Exp $ */
 
 #include <config.h>
 #include "trio.h"
@@ -27,7 +27,7 @@
 
 #include "CatJob.h"
 #include "ArgV.h"
-#include "OutputJob.h"
+#include "FileCopyOutputJob.h"
 
 #define super CopyJobEnv
 
@@ -65,7 +65,7 @@ void CatJob::NextFile()
       return;
    }
 
-   FileCopyPeerFA *src_peer=FileCopyPeerFA::New(session,src,FA::RETRIEVE,false);
+   FileCopyPeerFA *src_peer=FileCopyPeerFA::New(session,src,FA::RETRIEVE);
    FileCopyPeer *dst_peer=new FileCopyPeerOutputJob(output);
 
    FileCopy *copier=FileCopy::New(src_peer,dst_peer,false);
@@ -82,15 +82,9 @@ void CatJob::NextFile()
    SetCopier(copier,src);
 }
 
-CatJob::~CatJob()
-{
-   delete output;
-}
-
 CatJob::CatJob(FileAccess *new_session,OutputJob *_output,ArgV *new_args)
-   : CopyJobEnv(new_session,new_args)
+   : CopyJobEnv(new_session,new_args), output(_output)
 {
-   output=_output;
    output->SetParentFg(this);
    ascii=false;
    auto_ascii=true;
@@ -117,10 +111,8 @@ CatJob::CatJob(FileAccess *new_session,OutputJob *_output,ArgV *new_args)
    }
 }
 
-void CatJob::ShowRunStatus(StatusLine *s)
+void CatJob::ShowRunStatus(const SMTaskRef<StatusLine>& s)
 {
    if(cp && cp->HasStatus() && output->ShowStatusLine(s))
-   {
       cp->ShowRunStatus(s);
-   }
 }
