@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-2007 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2010 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: CmdExec.h,v 1.70 2009/07/17 12:48:41 lav Exp $ */
+/* $Id: CmdExec.h,v 1.76 2011/06/10 09:36:43 lav Exp $ */
 
 #ifndef CMDEXEC_H
 #define CMDEXEC_H
@@ -59,7 +59,7 @@ public:
 
 extern CmdFeeder *lftp_feeder;	 // feeder to use after 'lftp' command
 
-class CmdExec : public SessionJob
+class CmdExec : public SessionJob, public ResClient
 {
 public:
 // current command data
@@ -73,6 +73,7 @@ private:
    Buffer cmd_buf;
    bool partial_cmd;
    int alias_field; // length of expanded alias (and ttl for used_aliases)
+   int failed_exit_code;
 
    TouchedAlias *used_aliases;
    void free_used_aliases();
@@ -146,14 +147,15 @@ private:
    void ReuseSavedSession();
    void RevertToSavedSession();
 
+   void init(LocalDirectory *c);
+
 public:
    void FeedCmd(const char *c);
    void FeedArgV(const ArgV *,int start=0);
    void PrependCmd(const char *c);
    void ExecParsed(ArgV *a,FDStream *o=0,bool b=false);
-   static int unquote(char *buf,const char *str);  // returns buf length
-   static const char *unquote(const char *str);
-   static bool needs_quotation(const char *buf);
+   static bool needs_quotation(const char *buf,int len);
+   static bool needs_quotation(const char *buf) { return needs_quotation(buf,strlen(buf)); }
    static bool quotable(char c,char in_quotes);
    static bool is_space(char c) { return c==' ' || c=='\t'; }
    static bool is_quote(char c) { return c=='"' || c=='\''; }
@@ -164,16 +166,18 @@ public:
    bool WriteCmds(int fd) const;
    bool ReadCmds(int fd); // does not clear queue before reading (appends)
 
+   void AddNewJob(Job *new_job);
    void SuspendJob(Job *j);
 
    CmdExec(FileAccess *s,LocalDirectory *c);
+   CmdExec(const CmdExec *parent);
    ~CmdExec();
 
    bool Idle();	// when we have no command running and command buffer is empty
    int Done();
-   int ExitCode() { return exit_code; }
+   int ExitCode() { return failed_exit_code ? failed_exit_code : exit_code; }
    int Do();
-   void PrintStatus(int,const char *prefix="\t");
+   xstring& FormatStatus(xstring&,int,const char *prefix="\t");
    void ShowRunStatus(const SMTaskRef<StatusLine>& s);
    int AcceptSig(int sig);
 
@@ -233,6 +237,7 @@ public:
    Job *builtin_glob();
    Job *builtin_queue();
    Job *builtin_queue_edit();
+   Job *builtin_local();
 
    Job *default_cmd();
 
