@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-2008 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2010 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileAccess.cc,v 1.164 2010/02/05 14:34:48 lav Exp $ */
+/* $Id: FileAccess.cc,v 1.168 2011/03/25 14:35:41 lav Exp $ */
 
 #include <config.h>
 
@@ -207,11 +207,11 @@ void FileAccess::Close()
    ClearError();
 }
 
-void FileAccess::Rename(const char *f,const char *f1)
+void FileAccess::Open2(const char *f,const char *f1,open_mode o)
 {
    Close();
    file1.set(f1);
-   Open(f,RENAME);
+   Open(f,o);
 
    cache->TreeChanged(this,file);
    cache->FileChanged(this,file);
@@ -563,7 +563,7 @@ void FileAccess::Chmod(const char *file,int m)
 
 void FileAccess::SetError(int ec,const char *e)
 {
-   if(ec==SEE_ERRNO)
+   if(ec==SEE_ERRNO && !saved_errno)
       saved_errno=errno;
    if(ec==NO_FILE && file && file[0] && !strstr(e,file))
       error.vset(e," (",file.get(),")",NULL);
@@ -574,6 +574,7 @@ void FileAccess::SetError(int ec,const char *e)
 
 void FileAccess::ClearError()
 {
+   saved_errno=0;
    error_code=OK;
    error.set(0);
 }
@@ -861,8 +862,7 @@ FileAccess *FileAccess::Protocol::NewSession(const char *proto)
 
 #ifdef WITH_MODULES
 #define PROTO_PREFIX "proto-"
-   char *mod=(char*)alloca(strlen(PROTO_PREFIX)+strlen(proto)+1);
-   sprintf(mod,"%s%s",PROTO_PREFIX,proto);
+   const char *mod=xstring::cat(PROTO_PREFIX,proto,NULL);
    void *map=module_load(mod,0,0);
    if(map==0)
    {
@@ -947,11 +947,7 @@ void FileAccess::Path::Set(const Path *o)
 void FileAccess::Path::Change(const char *new_path,bool new_is_file,const char *new_path_enc,int new_device_prefix_len)
 {
    if(!new_path && new_path_enc)
-   {
-      char *np=alloca_strdup(new_path_enc);
-      url::decode_string(np);
-      new_path=np;
-   }
+      new_path=url::decode(new_path_enc);
    if(!new_path || !*new_path)
       return;
    const char *bn=basename_ptr(new_path);
@@ -996,12 +992,10 @@ void FileAccess::Path::Change(const char *new_path,bool new_is_file,const char *
 	 if(!path[0])
 	    path.set("~");
       }
-      char *newcwd=string_alloca(xstrlen(path)+xstrlen(new_path)+2);
       if(last_char(path)=='/')
-	 sprintf(newcwd,"%s%s",path.get(),new_path);
+	 new_path=xstring::format("%s%s",path.get(),new_path);
       else
-	 sprintf(newcwd,"%s/%s",path.get(),new_path);
-      new_path=newcwd;
+	 new_path=xstring::format("%s/%s",path.get(),new_path);
    }
    path.set(new_path);
    device_prefix_len=new_device_prefix_len;
