@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1999-2005 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1999-2010 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileCopy.cc,v 1.155 2010/01/21 08:22:26 lav Exp $ */
+/* $Id: FileCopy.cc,v 1.162 2011/04/22 15:43:11 lav Exp $ */
 
 /* FileCopyPeer behaviour:
     1) when suspended, does nothing
@@ -629,7 +629,10 @@ void FileCopy::LogTransfer()
       return;
    if(!transfer_log)
    {
-      int fd=open(dir_file(get_lftp_home(),"transfer_log"),O_WRONLY|O_APPEND|O_CREAT,0600);
+      const char *fname = ResMgr::Query("xfer:log-file", 0);
+      if(!fname || !*fname)
+	 fname = dir_file(get_lftp_home(),"transfer_log");
+      int fd = open(fname,O_WRONLY|O_APPEND|O_CREAT,0600);
       if(fd==-1)
 	 return;
       transfer_log=new Log;
@@ -641,7 +644,8 @@ void FileCopy::LogTransfer()
    long long range_limit=GetRangeLimit();
    if(range_limit==FILE_END)
       range_limit=get->GetPos();
-   transfer_log->Format(0,"%s -> %s %lld-%lld %s\n",src,dst,
+   transfer_log->Format(0,"%s -> %s %lld-%lld %s\n",
+      url::remove_password(src),url::remove_password(dst),
       (long long)GetRangeStart(),range_limit,
       Speedometer::GetStr(GetBytesCount()/GetTimeSpent()).get());
 }
@@ -729,6 +733,7 @@ FileCopyPeer::FileCopyPeer(dir_t m) : IOBuffer(m)
    can_seek0=false;
    date_set=false;
    do_set_date=true;
+   do_verify=true;
    ascii=false;
    range_start=0;
    range_limit=FILE_END;
@@ -834,8 +839,10 @@ int FileCopyPeerFA::Do()
 	    fxp_eof:
 	       // FIXME: set date for real.
 	       date_set=true;
-	       if(!verify)
+	       if(!verify && do_verify)
 		  verify=new FileVerificator(session,file);
+	       else
+		  done=true;
 	       return MOVED;
 	    }
 	    else if(res==FA::IN_PROGRESS)
@@ -848,6 +855,11 @@ int FileCopyPeerFA::Do()
 	       {
 		  try_time=session->GetTryTime();
 		  retries=session->GetRetries();
+		  off_t pos=session->GetRealPos();
+		  if(upload_watermark<pos) {
+		     upload_watermark=pos;
+		     retries=-1;
+		  }
 		  Log::global->Format(10,"try_time=%ld, retries=%d\n",(long)try_time,retries);
 		  session->Close();
 		  if(can_seek && seek_pos>0)
@@ -982,9 +994,9 @@ void FileCopyPeerFA::OpenSession()
    session->SetLimit(range_limit);
    if(mode==PUT)
    {
-      if(try_time!=0)
+      if(try_time!=NO_DATE)
 	 session->SetTryTime(try_time);
-      if(retries!=0)
+      if(retries>=0)
 	 session->SetRetries(retries+1);
       if(e_size!=NO_SIZE && e_size!=NO_SIZE_YET)
 	 session->SetSize(e_size);
@@ -1056,8 +1068,9 @@ int FileCopyPeerFA::Get_LL(int len)
 	    if(FAmode==FA::QUOTE_CMD)
 	       FAmode=FA::RETRIEVE;
 
-	    char *loc=alloca_strdup(loc_c);
+	    xstring& loc=xstring::get_tmp(loc_c);
 	    session->Close(); // loc_c is no longer valid.
+	    loc_c=0;
 
 	    ParsedURL u(loc,true);
 
@@ -1091,7 +1104,7 @@ int FileCopyPeerFA::Get_LL(int len)
 		  }
 	       }
 
-	       url::decode_string(loc);
+	       loc.url_decode();
 	       const char *slash=strrchr(file,'/');
 	       if(loc[0]!='/' && slash)
 	       {
@@ -1105,8 +1118,8 @@ int FileCopyPeerFA::Get_LL(int len)
 	    size=NO_SIZE_YET;
 	    date=NO_DATE_YET;
 
-	    try_time=0;
-	    retries=0;
+	    try_time=NO_DATE;
+	    retries=-1;
 	    current->Timeout(0); // retry with new location.
 	    return 0;
 	 }
@@ -1145,6 +1158,11 @@ int FileCopyPeerFA::Put_LL(const char *buf,int len)
       {
 	 try_time=session->GetTryTime();
 	 retries=session->GetRetries();
+	 off_t pos=session->GetRealPos();
+	 if(upload_watermark<pos) {
+	    upload_watermark=pos;
+	    retries=-1;
+	 }
 	 Log::global->Format(10,"try_time=%ld, retries=%d\n",(long)try_time,retries);
 	 session->Close();
 	 if(can_seek && seek_pos>0)
@@ -1198,8 +1216,9 @@ off_t FileCopyPeerFA::GetRealPos()
 void FileCopyPeerFA::Init()
 {
    fxp=false;
-   try_time=0;
-   retries=0;
+   try_time=NO_DATE;
+   retries=-1;
+   upload_watermark=0;
    redirections=0;
    can_seek=true;
    can_seek0=true;
@@ -1254,7 +1273,9 @@ FileCopyPeerFA *FileCopyPeerFA::New(const FileAccessRef& s,const char *url,int m
 }
 FileCopyPeer *FileCopyPeerFA::Clone()
 {
-   return new FileCopyPeerFA(session->Clone(),file,FAmode);
+   FileCopyPeerFA *c=new FileCopyPeerFA(session->Clone(),file,FAmode);
+   c->orig_url.set(orig_url);
+   return c;
 }
 
 // FileCopyPeerFDStream
@@ -1416,8 +1437,10 @@ int FileCopyPeerFDStream::Do()
 	    }
 	    if(stream && my_stream && !stream->Done())
 	       return m;
-	    if(!verify)
+	    if(!verify && do_verify)
 	       verify=new FileVerificator(stream);
+	    else
+	       done=true;
 	    return MOVED;
 	 }
 	 if(seek_pos==0)
