@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1998-2005 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1998-2010 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileSet.cc,v 1.70 2009/09/15 07:56:29 lav Exp $ */
+/* $Id: FileSet.cc,v 1.75 2011/04/29 04:58:27 lav Exp $ */
 
 #include <config.h>
 #include "FileSet.h"
@@ -451,13 +451,14 @@ int FileSet::FindGEIndByName(const char *name) const
       int m = (l + u) / 2;
       int cmp = strcmp(files[m]->name, name);
 
-      /* if files[m]->name >= name, update the upper bound: */
-      if (cmp >= 0)
+      /* if files[m]->name > name, update the upper bound: */
+      if (cmp > 0)
 	 u = m;
-
       /* if files[m]->name < name, update the lower bound: */
-      if (cmp < 0)
+      else if (cmp < 0)
 	 l = m+1;
+      else /* otherwise found exact match */
+	 return m;
    }
 
    return u;
@@ -682,7 +683,6 @@ check_again:
       t=FileInfo::DIRECTORY;
    else if(S_ISREG(st.st_mode))
       t=FileInfo::NORMAL;
-#ifdef HAVE_LSTAT
    else if(S_ISLNK(st.st_mode))
    {
       if(follow_symlinks)
@@ -693,7 +693,6 @@ check_again:
       }
       t=FileInfo::SYMLINK;
    }
-#endif
    else
       return;   // ignore other type files
 
@@ -712,10 +711,9 @@ check_again:
    SetUser(PasswdCache::LookupS(st.st_uid));
    SetGroup(GroupCache::LookupS(st.st_gid));
 
-#ifdef HAVE_LSTAT
    if(t==SYMLINK)
    {
-      char *buf=(char*)alloca(st.st_size+1);
+      char *buf=string_alloca(st.st_size+1);
       int res=readlink(name,buf,st.st_size);
       if(res!=-1)
       {
@@ -723,7 +721,6 @@ check_again:
 	 SetSymlink(buf);
       }
    }
-#endif /* HAVE_LSTAT */
 }
 
 /* parse_ls_line: too common procedure to make it protocol specific */
@@ -930,20 +927,20 @@ void FileInfo::MakeLongName()
    int mode1=(defined&MODE?mode:
       (filetype_c=='d'?0755:(filetype_c=='l'?0777:0644)));
 
-   char usergroup[33];
-   usergroup[0]=0;
-   if(defined&(USER|GROUP))
-      sprintf(usergroup,"%.16s%s%.16s",defined&USER?user:"?",
+   const char *usergroup="";
+   if(defined&(USER|GROUP)) {
+      usergroup=xstring::format("%.16s%s%.16s",defined&USER?user:"?",
 		  defined&GROUP?"/":"",defined&GROUP?group:"");
+   }
 
    int w=20-strlen(usergroup);
    if(w<1)
       w=1;
    char size_str[20];
    if(defined&SIZE)
-      sprintf(size_str,"%*lld",w,(long long)size);
+      snprintf(size_str,sizeof(size_str),"%*lld",w,(long long)size);
    else
-      sprintf(size_str,"%*s",w,"-");
+      snprintf(size_str,sizeof(size_str),"%*s",w,"-");
 
    const char *date_str="-";
    if(defined&DATE)
