@@ -7,7 +7,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -113,15 +113,15 @@ void FileSetOutput::print(FileSet &fs, const JobRef<OutputJob>& o) const
       }
 
       if((mode & SIZE) && (have&FileInfo::SIZE)) {
-	 char sz[128];
+	 char sz[LONGEST_HUMAN_READABLE + 2];
 	 if((f->filetype == FileInfo::NORMAL || !size_filesonly)
 	 && (f->defined&f->SIZE)) {
-	    char buffer[128];
-	    sprintf(sz, "%8s ",
+	    char buffer[LONGEST_HUMAN_READABLE + 1];
+	    snprintf(sz, sizeof(sz), "%8s ",
 	       human_readable (f->size, buffer, human_opts, 1,
 		  output_block_size? output_block_size:1024));
 	 } else {
-	    sprintf(sz, "%8s ", ""); /* pad */
+	    snprintf(sz, sizeof(sz), "%8s ", ""); /* pad */
 	 }
 	 c.add(sz, "");
       }
@@ -288,7 +288,7 @@ clsJob::clsJob(FA *s, ArgV *a, FileSetOutput *_opts, OutputJob *_output):
       args->Add("");
 
    output=_output;
-   output->SetParentFg(this);
+   AddWaiting(output);
 }
 
 int clsJob::Done()
@@ -414,9 +414,9 @@ void clsJob::ShowRunStatus(const SMTaskRef<StatusLine>& s)
 	 s->Show("%s", output->Status(s));
 }
 
-void clsJob::PrintStatus(int v,const char *prefix)
+xstring& clsJob::FormatStatus(xstring& s,int v,const char *prefix)
 {
-   Job::PrintStatus(v,prefix);
+   Job::FormatStatus(s,v,prefix);
 
    if(list_info)
    {
@@ -425,6 +425,7 @@ void clsJob::PrintStatus(int v,const char *prefix)
 	 curr = ".";
       const char *stat = list_info->Status();
       if(*stat)
-	 printf("%s`%s' %s\n", prefix, curr, stat);
+	 s.appendf("%s`%s' %s\n", prefix, curr, stat);
    }
+   return s;
 }
