@@ -1,11 +1,11 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 2002-2006 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 2002-2007 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -49,8 +49,8 @@ public:
    };
    int mode;
 
-   char *pat;
-   char *time_fmt;
+   xstring_c pat;
+   xstring_c time_fmt;
 
    bool basenames;
    bool showdots;
@@ -64,36 +64,38 @@ public:
    bool list_directories;
    bool need_exact_time;
    int output_block_size;
+   int human_opts;
 
    FileSet::sort_e sort;
    FileSetOutput(): classify(0), width(0), color(false), mode(NONE),
-      pat(NULL), time_fmt(0), basenames(false), showdots(false),
+      basenames(false), showdots(false),
       quiet(false), patterns_casefold(false), sort_casefold(false), sort_reverse(false),
       sort_dirs_first(false), size_filesonly(false), single_column(false),
-      list_directories(false), need_exact_time(false), output_block_size(0), sort(FileSet::BYNAME) { }
-   ~FileSetOutput() { xfree(pat); xfree(time_fmt); }
-   FileSetOutput(const FileSetOutput &cp);
-   const FileSetOutput &operator = (const FileSetOutput &cp);
+      list_directories(false), need_exact_time(false), output_block_size(0),
+      human_opts(0), sort(FileSet::BYNAME) { }
 
    void long_list();
    void config(const OutputJob *fd);
-   const char *parse_argv(ArgV *a);
-   static const char *ValidateArgv(char **s);
+   const char *parse_argv(const Ref<ArgV>& a);
+   const char *parse_res(const char *res);
+   static const char *ValidateArgv(xstring_c *s);
    int Need() const;
 
-   void print(FileSet &fs, OutputJob *o) const;
+   void print(FileSet &fs, const JobRef<OutputJob>& o) const;
 };
 
 /* Job interface to FileSetOutput */
 class clsJob : public SessionJob
 {
-   OutputJob *output;
-   FileSetOutput fso;
-   ArgV *args;
-   ListInfo *list_info;
-   char *dir, *mask;
+   JobRef<OutputJob> output;
+   Ref<FileSetOutput> fso;
+   Ref<ArgV> args;
+   SMTaskRef<ListInfo> list_info;
+   xstring_c dir;
+   xstring_c mask;
    bool done;
    bool use_cache;
+   bool error;
 
    /* element in args we're currently processing */
    int num;
@@ -101,8 +103,7 @@ class clsJob : public SessionJob
    enum { INIT, START_LISTING, GETTING_LIST_INFO, DONE } state;
 
 public:
-   clsJob(FA *s, ArgV *a, const FileSetOutput &_opts, OutputJob *output);
-   ~clsJob();
+   clsJob(FA *s, ArgV *a, FileSetOutput *_opts, OutputJob *output);
    int Done();
    int Do();
 
@@ -112,10 +113,10 @@ public:
    void Bg() { session->SetPriority(0); output->Bg(); }
    void SuspendInternal();
    void ResumeInternal();
-   int ExitCode() { return output->Error()? 1:0; }
+   int ExitCode() { return error||output->Error() ? 1 : 0; }
 
-   void ShowRunStatus(StatusLine *s);
-   void PrintStatus(int v,const char *);
+   void ShowRunStatus(const SMTaskRef<StatusLine>&);
+   xstring& FormatStatus(xstring&,int v,const char *);
 };
 
 #endif
