@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FindJob.h,v 1.23 2007/09/10 10:08:30 lav Exp $ */
+/* $Id: FindJob.h,v 1.25 2011/04/29 04:58:27 lav Exp $ */
 
 #ifndef FINDJOB_H
 #define FINDJOB_H
@@ -106,7 +106,7 @@ public:
    ~FinderJob();
 
    void ShowRunStatus(const SMTaskRef<StatusLine>&);
-   void PrintStatus(int v,const char *);
+   xstring& FormatStatus(xstring&,int,const char *);
 
    void BeQuiet() { quiet=true; }
    void SetExclude(PatternSet *p) { exclude = p; }
