@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1999-2006 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1999-2010 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: Http.cc,v 1.236 2009/07/17 12:52:26 lav Exp $ */
+/* $Id: Http.cc,v 1.242 2011/04/29 04:58:27 lav Exp $ */
 
 #include <config.h>
 #include "trio.h"
@@ -31,6 +31,7 @@
 #include <time.h>
 #include <fnmatch.h>
 #include <locale.h>
+#include <assert.h>
 #include "Http.h"
 #include "ResMgr.h"
 #include "log.h"
@@ -259,9 +260,7 @@ void Http::SendMethod(const char *method,const char *efile)
    && (strlen(efile)<7 || strncmp(efile+strlen(efile)-7,";type=",6))
    && QueryBool("use-type",hostname))
    {
-      char *pfile=alloca_strdup2(efile,7);
-      sprintf(pfile,"%s;type=%c",efile,ascii?'a':'i');
-      efile=pfile;
+      efile=xstring::format("%s;type=%c",efile,ascii?'a':'i');
    }
 
    /*
@@ -369,6 +368,8 @@ bool Http::ModeSupported()
    case QUOTE_CMD:
    case LIST:
    case CHANGE_MODE:
+   case LINK:
+   case SYMLINK:
       return false;
    case CONNECT_VERIFY:
    case RETRIEVE:
@@ -524,6 +525,8 @@ void Http::SendRequest(const char *connection,const char *f)
 
    case LIST:
    case CHANGE_MODE:
+   case LINK:
+   case SYMLINK:
       abort(); // unsupported
 
    case RETRIEVE:
@@ -564,12 +567,11 @@ void Http::SendRequest(const char *connection,const char *f)
       }
       if(entity_date!=NO_DATE)
       {
-	 char d[256];
 	 static const char weekday_names[][4]={
 	    "Sun","Mon","Tue","Wed","Thu","Fri","Sat"
 	 };
-	 struct tm *t=gmtime(&entity_date);
-	 sprintf(d,"%s, %2d %s %04d %02d:%02d:%02d GMT",
+	 const struct tm *t=gmtime(&entity_date);
+	 const char *d=xstring::format("%s, %2d %s %04d %02d:%02d:%02d GMT",
 	    weekday_names[t->tm_wday],t->tm_mday,month_names[t->tm_mon],
 	    t->tm_year+1900,t->tm_hour,t->tm_min,t->tm_sec);
 	 Send("Last-Modified: %s\r\n",d);
@@ -945,26 +947,22 @@ int Http::Do()
 	    const char *scan=file+5;
 	    while(*scan==' ')
 	       scan++;
-	    char *url=string_alloca(5+xstrlen(hostname)*3+1+xstrlen(portname)*3
-				    +1+xstrlen(cwd)*3+1+strlen(scan)+1);
-	    strcpy(url,https?"https://":"http://");
-	    url::encode_string(hostname,url+strlen(url),URL_HOST_UNSAFE);
+	    file_url.set(https?"https://":"http://");
+	    file_url.append_url_encoded(hostname,URL_HOST_UNSAFE);
 	    if(portname)
 	    {
-	       strcat(url,":");
-	       url::encode_string(portname,url+strlen(url),URL_PORT_UNSAFE);
+	       file_url.append(':');
+	       file_url.append_url_encoded(portname,URL_PORT_UNSAFE);
 	    }
 	    if(*scan!='/' && cwd)
 	    {
 	       if(cwd[0]!='/')
-		  strcat(url,"/");
-	       url::encode_string(cwd,url+strlen(url),URL_PATH_UNSAFE);
+		  file_url.append('/');
+	       file_url.append_url_encoded(cwd,URL_PATH_UNSAFE);
 	    }
 	    if(*scan!='/')
-	       strcat(url,"/");
-	    strcat(url,scan);
-
-	    file_url.set(url);
+	       file_url.append('/');
+	    file_url.append(scan);
 	    file_url.truncate_at(' ');
 
 	    scan=strchr(scan,' ');
@@ -1024,18 +1022,18 @@ int Http::Do()
       sock=SocketCreateTCP(peer[peer_curr].sa.sa_family);
       if(sock==-1)
       {
+	 saved_errno=errno;
 	 if(peer_curr+1<peer.count())
 	 {
 	    peer_curr++;
 	    retries--;
 	    return MOVED;
 	 }
-	 if(NonFatalError(errno))
+	 if(NonFatalError(saved_errno))
 	    return m;
-	 char str[256];
-	 sprintf(str,_("cannot create socket of address family %d"),
-			peer[peer_curr].sa.sa_family);
-	 SetError(SEE_ERRNO,str);
+	 SetError(SEE_ERRNO,xstring::format(
+	    _("cannot create socket of address family %d"),
+	    peer[peer_curr].sa.sa_family));
 	 return MOVED;
       }
 
@@ -1043,10 +1041,11 @@ int Http::Do()
       res=SocketConnect(sock,&peer[peer_curr]);
       if(res==-1 && errno!=EINPROGRESS)
       {
+	 saved_errno=errno;
 	 NextPeer();
-	 LogError(0,"connect: %s\n",strerror(errno));
+	 LogError(0,"connect: %s\n",strerror(saved_errno));
 	 Disconnect();
-	 if(NotSerious(errno))
+	 if(NotSerious(saved_errno))
 	    return MOVED;
 	 goto system_error;
       }
@@ -1091,11 +1090,9 @@ int Http::Do()
 	 if(proxy && https)
 	 {
 	    // have to setup a tunnel.
-	    char *ehost=string_alloca(strlen(hostname)*3+1);
+	    const char *ehost=url::encode(hostname,URL_HOST_UNSAFE);
 	    const char *port_to_use=portname?portname.get():HTTPS_DEFAULT_PORT;
-	    char *eport=string_alloca(strlen(port_to_use)*3+1);
-	    url::encode_string(hostname,ehost,URL_HOST_UNSAFE);
-	    url::encode_string(port_to_use,eport,URL_PORT_UNSAFE);
+	    const char *eport=url::encode(port_to_use,URL_PORT_UNSAFE);
 	    Send("CONNECT %s:%s HTTP/1.1\r\n\r\n",ehost,eport);
 	    tunnel_state=TUNNEL_WAITING;
 	    state=RECEIVING_HEADER;
@@ -1378,7 +1375,7 @@ int Http::Do()
 
       if(!H_20X(status_code))
       {
-	 char *err=string_alloca(strlen(status)+strlen(file)+strlen(cwd)+xstrlen(location)+20);
+	 xstring& err=xstring::get_tmp();
 	 int code=NO_FILE;
 
 	 if(H_REDIRECTED(status_code))
@@ -1416,7 +1413,7 @@ int Http::Do()
 	       }
 	       location.set(new_location);
 	    }
-	    sprintf(err,"%s (%s -> %s)",status+status_consumed,file.get(),
+	    err.setf("%s (%s -> %s)",status+status_consumed,file.get(),
 				    location?location.get():"nowhere");
 	    code=FILE_MOVED;
 	 }
@@ -1445,9 +1442,9 @@ int Http::Do()
 	       closure=last_method;
 	    }
 	    if(closure && closure[0])
-	       sprintf(err,"%s (%s)",status+status_consumed,closure);
+	       err.setf("%s (%s)",status+status_consumed,closure);
 	    else
-	       sprintf(err,"%s (%s%s)",status+status_consumed,cwd.path.get(),
+	       err.setf("%s (%s%s)",status+status_consumed,cwd.path.get(),
 				       (last_char(cwd)=='/')?"":"/");
 	 }
 	 state=RECEIVING_BODY;
@@ -1523,7 +1520,8 @@ int Http::Do()
    return m;
 
 system_error:
-   if(NonFatalError(errno))
+   assert(saved_errno!=0);
+   if(NonFatalError(saved_errno))
       return m;
    SetError(SEE_ERRNO,0);
    Disconnect();
@@ -2077,15 +2075,11 @@ void Http::SetCookie(const char *value_const)
       }
    }
 
-   char *closure=string_alloca(strlen(domain)+xstrlen(path)+32);
-   strcpy(closure,domain);
+   xstring closure(domain);
    if(path && path[0] && strcmp(path,"/"))
-   {
-      strcat(closure,";path=");
-      strcat(closure,path);
-   }
+      closure.append(";path=").append(path);
    if(secure)
-      strcat(closure,";secure");
+      closure.append(";secure");
 
    xstring c(Query("cookie",closure));
    CookieMerge(c,value_const);
@@ -2184,9 +2178,11 @@ void Http::LogErrorText()
       return;
    buf[size]=0;
    remove_tags(buf);
-   for(char *line=strtok(buf,"\n"); line; line=strtok(0,"\n"))
+   for(char *line=strtok(buf,"\n"); line; line=strtok(0,"\n")) {
+      rtrim(line);
       if(*line)
 	 Log::global->Format(4,"<--* %s\n",line);
+   }
 }
 
 
