@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-2007 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2010 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: MirrorJob.cc,v 1.168 2008/11/27 05:56:22 lav Exp $ */
+/* $Id: MirrorJob.cc,v 1.178 2011/04/29 04:58:28 lav Exp $ */
 
 #include <config.h>
 
@@ -32,6 +32,7 @@
 #include "MirrorJob.h"
 #include "CmdExec.h"
 #include "rmJob.h"
+#include "mvJob.h"
 #include "ChmodJob.h"
 #include "misc.h"
 #include "plural.h"
@@ -45,7 +46,7 @@
    Log::global->Format(11,"mirror(%p) enters state %s\n", this, #s); } while(0)
 #define waiting_num waiting.count()
 
-void  MirrorJob::PrintStatus(int v,const char *tab)
+xstring& MirrorJob::FormatStatus(xstring& s,int v,const char *tab)
 {
    if(Done())
       goto final;
@@ -63,60 +64,61 @@ void  MirrorJob::PrintStatus(int v,const char *tab)
       break;
 
    case(MAKE_TARGET_DIR):
-      printf("\tmkdir `%s' [%s]\n",target_dir.get(),target_session->CurrentStatus());
+      s.appendf("\tmkdir `%s' [%s]\n",target_dir.get(),target_session->CurrentStatus());
       break;
 
    case(CHANGING_DIR_SOURCE):
    case(CHANGING_DIR_TARGET):
       if(target_session->IsOpen())
-	 printf("\tcd `%s' [%s]\n",target_dir.get(),target_session->CurrentStatus());
+	 s.appendf("\tcd `%s' [%s]\n",target_dir.get(),target_session->CurrentStatus());
       if(source_session->IsOpen())
-	 printf("\tcd `%s' [%s]\n",source_dir.get(),source_session->CurrentStatus());
+	 s.appendf("\tcd `%s' [%s]\n",source_dir.get(),source_session->CurrentStatus());
       break;
 
    case(GETTING_LIST_INFO):
       if(target_list_info)
       {
 	 if(target_relative_dir)
-	    printf("\t%s: %s\n",target_relative_dir.get(),target_list_info->Status());
+	    s.appendf("\t%s: %s\n",target_relative_dir.get(),target_list_info->Status());
 	 else
-	    printf("\t%s\n",target_list_info->Status());
+	    s.appendf("\t%s\n",target_list_info->Status());
       }
       if(source_list_info)
       {
 	 if(source_relative_dir)
-	    printf("\t%s: %s\n",source_relative_dir.get(),source_list_info->Status());
+	    s.appendf("\t%s: %s\n",source_relative_dir.get(),source_list_info->Status());
 	 else
-	    printf("\t%s\n",source_list_info->Status());
+	    s.appendf("\t%s\n",source_list_info->Status());
       }
       break;
    }
-   return;
+   return s;
 
 final:
    if(stats.dirs>0)
-      printf(plural("%sTotal: %d director$y|ies$, %d file$|s$, %d symlink$|s$\n",
+      s.appendf(plural("%sTotal: %d director$y|ies$, %d file$|s$, %d symlink$|s$\n",
 		     stats.dirs,stats.tot_files,stats.tot_symlinks),
 	 tab,stats.dirs,stats.tot_files,stats.tot_symlinks);
    if(stats.new_files || stats.new_symlinks)
-      printf(plural("%sNew: %d file$|s$, %d symlink$|s$\n",
+      s.appendf(plural("%sNew: %d file$|s$, %d symlink$|s$\n",
 		     stats.new_files,stats.new_symlinks),
 	 tab,stats.new_files,stats.new_symlinks);
    if(stats.mod_files || stats.mod_symlinks)
-      printf(plural("%sModified: %d file$|s$, %d symlink$|s$\n",
+      s.appendf(plural("%sModified: %d file$|s$, %d symlink$|s$\n",
 		     stats.mod_files,stats.mod_symlinks),
 	 tab,stats.mod_files,stats.mod_symlinks);
    if(stats.bytes)
-      printf("%s%s\n",tab,CopyJob::FormatBytesTimeRate(stats.bytes,stats.time));
+      s.appendf("%s%s\n",tab,CopyJob::FormatBytesTimeRate(stats.bytes,stats.time));
    if(stats.del_dirs || stats.del_files || stats.del_symlinks)
-      printf(plural(flags&DELETE ?
+      s.appendf(plural(flags&DELETE ?
 	       "%sRemoved: %d director$y|ies$, %d file$|s$, %d symlink$|s$\n"
 	      :"%sTo be removed: %d director$y|ies$, %d file$|s$, %d symlink$|s$\n",
 	      stats.del_dirs,stats.del_files,stats.del_symlinks),
 	 tab,stats.del_dirs,stats.del_files,stats.del_symlinks);
    if(stats.error_count)
-      printf(plural("%s%d error$|s$ detected\n",stats.error_count),
+      s.appendf(plural("%s%d error$|s$ detected\n",stats.error_count),
 	       tab,stats.error_count);
+   return s;
 }
 
 void  MirrorJob::ShowRunStatus(const SMTaskRef<StatusLine>& s)
@@ -233,8 +235,7 @@ void  MirrorJob::HandleFile(FileInfo *file)
 	    	((file->defined&file->DATE) && (old->defined&old->DATE)
 	    	&& file->date + file->date.ts_prec < old->date - old->date.ts_prec))
 	    && (file->defined&file->SIZE) && (old->defined&old->SIZE)
-	    && file->size >= old->size
-	    && !use_pget)
+	    && file->size >= old->size)
 	    {
 	       cont_this=true;
 	       if(target_is_local && !script_only)
@@ -419,8 +420,33 @@ void  MirrorJob::HandleFile(FileInfo *file)
 
 	 if(!target_is_local)
 	 {
-	    // can't create symlink remotely (FIXME?)
-	    goto skip;
+	    if(script)
+	    {
+	       ArgV args("ln");
+	       args.Append("-s");
+	       args.Append(file->symlink);
+	       args.Append(target_name);
+	       xstring_ca cmd(args.CombineQuoted());
+	       fprintf(script,"%s\n",cmd.get());
+	       if(script_only)
+		  goto skip;
+	    }
+	    bool remove_target=false;
+	    FileInfo *old=target_set->FindByName(file->name);
+	    if(old && !to_rm_mismatched->FindByName(file->name))
+	    {
+	       Report(_("Removing old file `%s'"),
+			dir_file(target_relative_dir,file->name));
+	       remove_target=true;
+	       stats.mod_symlinks++;
+	    }
+	    else
+	       stats.new_symlinks++;
+	    mvJob *j=new mvJob(target_session->Clone(),file->symlink,target_name,FA::SYMLINK);
+	    if(remove_target)
+	       j->RemoveTargetFirst();
+	    AddWaiting(j);
+	    break;
 	 }
 
 	 if(script)
@@ -436,7 +462,6 @@ void  MirrorJob::HandleFile(FileInfo *file)
 	       goto skip;
 	 }
 
-#ifdef HAVE_LSTAT
 	 if(file->defined&file->SYMLINK)
 	 {
 	    struct stat st;
@@ -467,7 +492,6 @@ void  MirrorJob::HandleFile(FileInfo *file)
 	    if(res==-1)
 	       eprintf("mirror: symlink(%s): %s\n",target_name,strerror(errno));
 	 }
-#endif /* LSTAT */
 	 break;
       }
    case FileInfo::UNKNOWN:
@@ -553,8 +577,7 @@ void MirrorJob::HandleChdir(FileAccessRef& session, int &redirections)
 
 	    if(!u.proto)
 	    {
-	       url::decode_string(loc);
-	       session->Chdir(loc);
+	       session->Chdir(url::decode(loc));
 	       return;
 	    }
 	    session=FA::New(&u);
@@ -982,9 +1005,7 @@ int   MirrorJob::Do()
 	 if(script)
 	 {
 	    ArgV args("chmod");
-	    char m[16];
-	    sprintf(m,"%03lo",(unsigned long)(file->mode&~mode_mask));
-	    args.Append(m);
+	    args.Append(xstring::format("%03lo",(unsigned long)(file->mode&~mode_mask)));
 	    args.Append(target_session->GetFileURL(file->name));
 	    xstring_ca cmd(args.CombineQuoted());
 	    fprintf(script,"%s\n",cmd.get());
@@ -1169,12 +1190,12 @@ void MirrorJob::Report(const char *fmt,...)
 }
 
 extern "C" {
-#include "getdate.h"
+#include "parse-datetime.h"
 }
 void MirrorJob::SetNewerThan(const char *f)
 {
    struct timespec ts;
-   if(get_date(&ts,f,0))
+   if(parse_datetime(&ts,f,0))
    {
       newer_than=ts.tv_sec;
       return;
@@ -1190,7 +1211,7 @@ void MirrorJob::SetNewerThan(const char *f)
 void MirrorJob::SetOlderThan(const char *f)
 {
    struct timespec ts;
-   if(get_date(&ts,f,0))
+   if(parse_datetime(&ts,f,0))
    {
       older_than=ts.tv_sec;
       return;
@@ -1271,12 +1292,15 @@ bool MirrorJob::Statistics::HaveSomethingDone(int flags)
    return new_files|mod_files|(del_files*del)|new_symlinks|mod_symlinks|(del_symlinks*del)|(del_dirs*del);
 }
 
-char *MirrorJob::SetScriptFile(const char *n)
+const char *MirrorJob::SetScriptFile(const char *n)
 {
    script_name.set(n);
    if(strcmp(n,"-"))
    {
       script=fopen(n,"w");
+      if(!script)
+	 return xstring::format("%s: %s",n,strerror(errno));
+      setvbuf(script,NULL,_IOLBF,0);
       script_needs_closing=true;
    }
    else
@@ -1284,9 +1308,7 @@ char *MirrorJob::SetScriptFile(const char *n)
       script=stdout;
       script_needs_closing=false;
    }
-   if(script)
-      return 0;
-   return xasprintf("%s: %s",n,strerror(errno));
+   return 0;
 }
 
 void MirrorJob::SetOnChange(const char *oc)
@@ -1389,6 +1411,7 @@ CMD(mirror)
    int	 use_pget=ResMgr::Query("mirror:use-pget-n",0);
    bool	 reverse=false;
    bool	 script_only=false;
+   bool	 no_empty_dirs=ResMgr::QueryBool("mirror:no-empty-dirs",0);
    const char *script_file=0;
    const char *on_change=0;
 
@@ -1402,7 +1425,7 @@ CMD(mirror)
       flags|=MirrorJob::RETR_SYMLINKS;
 
    args->rewind();
-   while((opt=args->getopt_long("esi:x:I:X:nrpcRvN:LPa",mirror_opts,0))!=EOF)
+   while((opt=args->getopt_long("esi:x:I:X:nrpcRvN:LP:a",mirror_opts,0))!=EOF)
    {
       switch(opt)
       {
@@ -1568,6 +1591,7 @@ CMD(mirror)
 	 flags|=MirrorJob::ONLY_EXISTING;
 	 break;
       case(OPT_NO_EMPTY_DIRS):
+	 no_empty_dirs=true;
 	 flags|=MirrorJob::NO_EMPTY_DIRS|MirrorJob::DEPTH_FIRST;
 	 break;
       case(OPT_DEPTH_FIRST):
@@ -1634,11 +1658,7 @@ CMD(mirror)
 	    // user wants source dir name appended.
 	    const char *base=basename_ptr(source_dir);
 	    if(base[0]!='/' && strcmp(base,basename_ptr(arg)))
-	    {
-	       char *target_dir1=alloca_strdup2(target_dir,strlen(base));
-	       strcat(target_dir1,base);
-	       target_dir=target_dir1;
-	    }
+	       target_dir=xstring::cat(target_dir,base,NULL);
 	 }
       }
       else
@@ -1666,6 +1686,9 @@ CMD(mirror)
 	 target_session=parent->session->Clone();
    }
 
+   if(no_empty_dirs)
+      flags|=MirrorJob::NO_EMPTY_DIRS|MirrorJob::DEPTH_FIRST;
+
    Ref<MirrorJob> j(new MirrorJob(0,source_session.borrow(),target_session.borrow(),source_dir,target_dir));
    j->SetFlags(flags,1);
    j->SetVerbose(verbose);
@@ -1693,10 +1716,11 @@ CMD(mirror)
 
    if(script_file)
    {
-      xstring_ca err(j->SetScriptFile(script_file));
+      const char *err=j->SetScriptFile(script_file);
       if(err)
       {
-	 eprintf("%s: %s\n",args->a0(),err.get());
+	 eprintf("%s: %s\n",args->a0(),err);
+	 j->DeleteLater();
 	 return 0;
       }
    }
