@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-2007 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2010 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: MirrorJob.h,v 1.54 2008/11/27 05:56:22 lav Exp $ */
+/* $Id: MirrorJob.h,v 1.57 2011/04/29 04:58:28 lav Exp $ */
 
 #ifndef MIRRORJOB_H
 #define MIRRORJOB_H
@@ -177,7 +177,7 @@ public:
    int	 Do();
    int	 Done() { return state==DONE; }
    void	 ShowRunStatus(const SMTaskRef<StatusLine>&);
-   void	 PrintStatus(int v,const char *);
+   xstring& FormatStatus(xstring&,int v,const char *);
    void	 SayFinal() { PrintStatus(0,""); }
    int	 ExitCode() { return stats.error_count; }
 
@@ -203,7 +203,7 @@ public:
    void Fg();
    void Bg();
 
-   char	 *SetScriptFile(const char *n);
+   const char *SetScriptFile(const char *n);
    void	 ScriptOnly(bool yes=true)
       {
 	 script_only=yes;
