@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: PtyShell.cc,v 1.15 2007/08/02 11:28:00 lav Exp $ */
+/* $Id: PtyShell.cc,v 1.17 2008/11/27 05:56:25 lav Exp $ */
 
 #include <config.h>
 #include "trio.h"
@@ -196,8 +196,6 @@ out:
 
 void PtyShell::Init()
 {
-   a=0;
-   w=0;
    oldcwd.set_allocated(xgetcwd());
    pg=0;
    closed=false;
@@ -218,27 +216,22 @@ PtyShell::PtyShell(const char *filter)
 }
 
 PtyShell::PtyShell(ArgV *a1)
-   : FDStream(-1,0)
+   : FDStream(-1,0), a(a1)
 {
    Init();
-   a=a1;
    name.set_allocated(a->Combine());
 }
 
 PtyShell::~PtyShell()
 {
-   delete a;
    if(fd!=-1)
-   {
       close(fd);
-      fd=-1;
-   }
    if(pipe_in!=-1)
       close(pipe_in);
    if(pipe_out!=-1)
       close(pipe_out);
    if(w)
-      w->Auto();
+      w.borrow()->Auto();
 }
 
 bool PtyShell::Done()
