@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-2005 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2010 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: ResMgr.cc,v 1.76 2009/10/23 06:20:34 lav Exp $ */
+/* $Id: ResMgr.cc,v 1.82 2011/05/10 05:53:20 lav Exp $ */
 
 #include <config.h>
 
@@ -538,7 +538,8 @@ bool ResMgr::Resource::ClosureMatch(const char *cl_data)
       return true;
    if(!(closure && cl_data))
       return false;
-   return 0==fnmatch(closure,cl_data,FNM_PATHNAME);
+   return 0==fnmatch(closure,cl_data,FNM_PATHNAME)
+      || (closure[0]=='*' && closure[1]=='.' && !strcmp(closure+2,cl_data));
 }
 
 const char *ResMgr::QueryNext(const char *name,const char **closure,Resource **ptr)
@@ -785,28 +786,16 @@ const char *Range::scale(long long *value,char suf)
    return 0;
 }
 
-#if !HAVE_DECL_RANDOM
-CDECL long random();
-CDECL void srandom(unsigned seed);
-#endif
-
 long long Range::Random()
 {
-   static bool init=false;
-   if(!init)
-   {
-      srandom(time(NULL)+getpid());
-      init=true;
-   }
+   random_init();
 
    if(no_start && no_end)
       return random();
    if(no_end)
       return start+random();
 
-   // interval [0;1)
-   double mult=random()/2147483648.0;
-   return start + (long long)((end-start+1)*mult);
+   return start + (long long)((end-start+1)*random01());
 }
 
 const char *ResMgr::RangeValidate(xstring_c *s)
@@ -825,8 +814,9 @@ const char *ResMgr::ERegExpValidate(xstring_c *s)
    int err=regcomp(&re,*s,REG_EXTENDED|REG_NOSUB);
    if(err)
    {
-      static char err_msg[128];
-      regerror(err,0,err_msg,sizeof(err_msg));
+      const int max_err_len=128;
+      char *err_msg=xstring::tmp_buf(max_err_len);
+      regerror(err,0,err_msg,max_err_len);
       return err_msg;
    }
    regfree(&re);
@@ -855,7 +845,7 @@ const char *ResMgr::IPv6AddrValidate(xstring_c *value)
 }
 #endif
 
-const char *ResMgr::FileAccessible(xstring_c *value,int mode,int want_dir)
+const char *ResMgr::FileAccessible(xstring_c *value,int mode,bool want_dir)
 {
    if(!**value)
       return 0;
@@ -872,7 +862,7 @@ const char *ResMgr::FileAccessible(xstring_c *value,int mode,int want_dir)
    if(stat(f,&st)<0)
       error=strerror(errno);
    else if(want_dir ^ S_ISDIR(st.st_mode))
-      error=strerror(want_dir?ENOTDIR:EISDIR);
+      error=strerror(errno=want_dir?ENOTDIR:EISDIR);
    else if(access(f,mode)<0)
       error=strerror(errno);
    else
@@ -889,7 +879,23 @@ const char *ResMgr::FileExecutable(xstring_c *value)
 }
 const char *ResMgr::DirReadable(xstring_c *value)
 {
-   return FileAccessible(value,R_OK|X_OK,1);
+   return FileAccessible(value,R_OK|X_OK,true);
+}
+const char *ResMgr::FileCreatable(xstring_c *value)
+{
+   if(!**value)
+      return 0;
+   const char *error=FileAccessible(value,W_OK,false);
+   if(error && errno!=ENOENT)
+      return error;
+   const char *bn=basename_ptr(*value);
+   xstring_c dir(dirname(*value));
+   if(!*dir)
+      dir.set_allocated(xgetcwd());
+   error=FileAccessible(&dir,X_OK|W_OK,true);
+   if(!error)  // dir may be expanded, combine it with base file name.
+      value->set(dir_file(dir,bn));
+   return error;
 }
 
 #ifdef HAVE_ICONV
@@ -948,9 +954,8 @@ ResValue ResClient::Query(const char *name,const char *closure) const
    if(!strchr(name,':'))
    {
       const char *prefix=ResPrefix();
-      char *fullname=(char*)alloca(3+strlen(prefix)+1+strlen(name)+1);
-      sprintf(fullname,"%s:%s",prefix,name);
-      name=fullname;
+      name=xstring::format("%s:%s",prefix,name);
+      name=alloca_strdup(name);
    }
    if(!closure)
       closure=ResClosure();
