@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1998-2008 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1998-2010 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: SleepJob.cc,v 1.42 2009/05/14 08:36:39 lav Exp $ */
+/* $Id: SleepJob.cc,v 1.46 2011/06/24 05:24:13 lav Exp $ */
 
 #include <config.h>
 #include <ctype.h>
@@ -105,15 +105,14 @@ const char *SleepJob::Status()
       NULL);
 }
 
-void SleepJob::PrintStatus(int,const char *prefix)
+xstring& SleepJob::FormatStatus(xstring& buf,int,const char *prefix)
 {
    if(repeat)
-   {
-      printf(_("\tRepeat count: %d\n"),repeat_count);
-   }
+      buf.appendf(_("\tRepeat count: %d\n"),repeat_count);
    const char *s=Status();
    if(s[0])
-      printf("\t%s\n",s);
+      buf.appendf("\t%s\n",s);
+   return buf;
 }
 void SleepJob::ShowRunStatus(const SMTaskRef<StatusLine>& s)
 {
@@ -239,13 +238,13 @@ Job *cmd_repeat(CmdExec *parent)
 }
 
 extern "C" {
-#include "getdate.h"
+#include "parse-datetime.h"
 }
 Job *cmd_at(CmdExec *parent)
 {
    int count=1;
    int cmd_start=0;
-   int date_len=0;
+   xstring date;
    for(;;)
    {
       const char *arg=args->getnext();
@@ -256,17 +255,21 @@ Job *cmd_at(CmdExec *parent)
 	 cmd_start=count+1;
 	 break;
       }
-      date_len+=strlen(arg)+1;
+      if(date)
+	 date.append(' ');
+      date.append(arg);
       count++;
    }
 
-   xstring_ca date(args->Combine(1));
-   date.truncate(date_len);
+   if(!date) {
+      eprintf(_("%s: date-time specification missed\n"),args->a0());
+      return 0;
+   }
 
    struct timespec ts;
-   if(!get_date(&ts,date,0))
+   if(!parse_datetime(&ts,date,0))
    {
-      eprintf("%s: %s\n",args->a0(),"date parse error");
+      eprintf(_("%s: date-time parse error\n"),args->a0());
       return 0;
    }
    time_t when=ts.tv_sec;
