@@ -1,11 +1,11 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-2005 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2010 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: StatusLine.cc,v 1.39 2007/08/02 11:28:00 lav Exp $ */
+/* $Id: StatusLine.cc,v 1.44 2011/04/29 04:58:28 lav Exp $ */
 
 #include <config.h>
 #include <sys/types.h>
@@ -154,7 +154,7 @@ void StatusLine::WriteTitle(const char *s, int fd) const
 
    const char *status_format = ResMgr::Query("cmd:term-status", getenv("TERM"));
 
-   xstring disp;
+   xstring &disp=xstring::get_tmp();
 
    /* If we have no format, and we have both tsl and fsl, use them: */
    if((!status_format || !*status_format) && to_status_line && from_status_line)
@@ -170,7 +170,7 @@ void StatusLine::update(const char *const *newstr,int newstr_height)
    if(not_term)
       return;
 
-   if(tcgetpgrp(fd)!=getpgrp())
+   if(!in_foreground_pgrp())
       return;
 
    /* Don't write blank titles into the title; let Clear() do that. */
@@ -258,17 +258,14 @@ void StatusLine::update(const char *const *newstr,int newstr_height)
 
 void StatusLine::WriteLine(const char *f,...)
 {
-   char *newstr=string_alloca(0x800+strlen(f));
-
    va_list v;
    va_start(v,f);
-   vsprintf(newstr,f,v);
+   xstring& newstr=xstring::vformat(f,v).append('\n');
    va_end(v);
 
    Clear();
 
-   strcat(newstr,"\n");
-   write(fd,newstr,strlen(newstr));
+   write(fd,newstr,newstr.length());
    update_delayed=false;
 }
 
