@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: Torrent.h,v 1.21 2010/06/09 12:42:26 lav Exp $ */
+/* $Id: Torrent.h,v 1.32 2011/05/10 05:55:14 lav Exp $ */
 
 #ifndef TORRENT_H
 #define TORRENT_H
@@ -30,6 +30,11 @@
 #include "network.h"
 #include "RateLimit.h"
 
+class FDCache;
+class TorrentBlackList;
+class Torrent;
+class TorrentPeer;
+
 class BitField : public xarray<unsigned char>
 {
    int bit_length;
@@ -50,7 +55,6 @@ public:
    void clear() { memset(buf,0,length()); }
 };
 
-class TorrentPeer;
 struct TorrentPiece
 {
    unsigned sources_count;	    // how many peers have the piece
@@ -65,15 +69,66 @@ struct TorrentPiece
    bool has_a_downloader() const;
 };
 
-class TorrentListener;
-class FDCache;
-class TorrentBlackList;
+class TorrentListener : public SMTask, protected ProtoLog, protected Networker
+{
+   Ref<Error> error;
+   int af;
+   int sock;
+   sockaddr_u addr;
+   Speedometer rate;
+   void FillAddress(int port);
+public:
+   TorrentListener(int a);
+   ~TorrentListener();
+   int Do();
+   int GetPort() const { return addr.port(); }
+   const char *GetAddress() const { return addr.address(); }
+};
+
+class TorrentTracker : public SMTask, protected ProtoLog
+{
+   friend class Torrent;
+
+   Torrent *parent;
+
+   xstring tracker_url;
+   FileAccessRef t_session;
+   Timer tracker_timer;
+   SMTaskRef<IOBuffer> tracker_reply;
+   xstring tracker_id;
+   bool started;
+   Ref<Error> error;
+   int tracker_no;
+
+   TorrentTracker(Torrent *p,const char *url);
+   void AddURL(const char *url);
+   int Do();
+
+   void Start();
+   void Shutdown();
+
+   void SendTrackerRequest(const char *event);
+   int HandleTrackerReply();
+
+   void SetError(const char *e) { error=new Error(-1,e,true); }
+   bool Failed() const { return error!=0; }
+   const char *ErrorText() const { return error->Text(); }
+
+public:
+   ~TorrentTracker() {}
+   const char *NextRequestIn() const {
+      return tracker_timer.TimeLeft().toString(
+	 TimeInterval::TO_STR_TRANSLATE|TimeInterval::TO_STR_TERSE);
+   }
+   const char *GetURL() const { return tracker_url; }
+   const char *Status() const;
+};
 
 class Torrent : public SMTask, protected ProtoLog, public ResClient
 {
    friend class TorrentPeer;
+   friend class TorrentDispatcher;
 
-   bool started;
    bool shutting_down;
    bool complete;
    bool end_game;
@@ -85,10 +140,18 @@ class Torrent : public SMTask, protected ProtoLog, public ResClient
    static const unsigned PEER_ID_LEN = 20;
    static xstring my_peer_id;
    static xstring my_key;
+   static xmap<Torrent*> torrents;
    static Ref<TorrentListener> listener;
+   static Ref<TorrentListener> listener_ipv6;
    static Ref<FDCache> fd_cache;
    static Ref<TorrentBlackList> black_list;
 
+   static Torrent *FindTorrent(const xstring& info_hash) { return torrents.lookup(info_hash); }
+   static void AddTorrent(Torrent *t) { torrents.add(t->GetInfoHash(),t); }
+   static void RemoveTorrent(Torrent *t) { torrents.remove(t->GetInfoHash()); }
+   static int GetTorrentsCount() { return torrents.count(); }
+   static void Dispatch(const xstring& info_hash,int s,const sockaddr_u *remote_addr,IOBuffer *recv_buf);
+
    xstring_c metainfo_url;
    FileAccessRef metainfo_fa;
    SMTaskRef<IOBuffer> metainfo_data;
@@ -102,25 +165,26 @@ class Torrent : public SMTask, protected ProtoLog, public ResClient
    void InitTranslation();
    void TranslateString(BeNode *node) const;
 
-   xstring tracker_url;
-   FileAccessRef t_session;
-   Timer tracker_timer;
-   SMTaskRef<IOBuffer> tracker_reply;
-   xstring tracker_id;
+   TaskRefArray<TorrentTracker> trackers;
+   bool TrackersDone() const;
+   bool TrackersFailed() const;
+   void StartTrackers() const;
+   void ShutdownTrackers() const;
+   void SendTrackersRequest(const char *e) const;
 
    bool single_file;
    unsigned piece_length;
    unsigned last_piece_length;
-   unsigned long long total_length;
    unsigned total_pieces;
-   Ref<BitField> my_bitfield;
-   unsigned long long total_left;
    unsigned complete_pieces;
+   Ref<BitField> my_bitfield;
 
    static const unsigned BLOCK_SIZE = 0x4000;
 
+   unsigned long long total_length;
    unsigned long long total_recv;
    unsigned long long total_sent;
+   unsigned long long total_left;
 
    void SetError(Error *);
    void SetError(const char *);
@@ -129,8 +193,6 @@ class Torrent : public SMTask, protected ProtoLog, public ResClient
    BeNode *Lookup(BeNode *d,const char *name,BeNode::be_type_t type) { return Lookup(d->dict,name,type); }
    BeNode *Lookup(Ref<BeNode>& d,const char *name,BeNode::be_type_t type) { return Lookup(d->dict,name,type); }
 
-   void SendTrackerRequest(const char *event);
-
    TaskRefArray<TorrentPeer> peers;
    RefArray<TorrentPiece> piece_info;
    static int PeersCompareActivity(const SMTaskRef<TorrentPeer> *p1,const SMTaskRef<TorrentPeer> *p2);
@@ -150,7 +212,8 @@ class Torrent : public SMTask, protected ProtoLog, public ResClient
 
    const char *FindFileByPosition(unsigned piece,unsigned begin,off_t *f_pos,off_t *f_tail) const;
    const char *MakePath(BeNode *p) const;
-   int OpenFile(const char *f,int m);
+   int OpenFile(const char *f,int m,off_t size=0);
+   void CloseFile(const char *f) const;
 
    void StoreBlock(unsigned piece,unsigned begin,unsigned len,const char *buf,TorrentPeer *src_peer);
    const xstring& RetrieveBlock(unsigned piece,unsigned begin,unsigned len);
@@ -161,6 +224,7 @@ class Torrent : public SMTask, protected ProtoLog, public ResClient
    RateLimit rate_limit;
    bool RateLow(RateLimit::dir_t dir) { return rate_limit.Relaxed(dir); }
 
+   int connected_peers_count;
    int active_peers_count;
    int complete_peers_count;
    int am_interested_peers_count;
@@ -196,14 +260,17 @@ class Torrent : public SMTask, protected ProtoLog, public ResClient
    void PeerBytesGot(int b) { PeerBytesUsed(b,RateLimit::GET); }
 
    static void BlackListPeer(const TorrentPeer *peer,const char *timeout);
+   TorrentPeer *FindPeerById(const xstring& p_id);
 
 public:
+   static void ClassInit();
+
    Torrent(const char *mf,const char *cwd,const char *output_dir);
 
    int Do();
-   int Done();
+   int Done() const;
 
-   const char *Status();
+   const xstring& Status();
 
    const Error *GetInvalidCause() const { return invalid_cause; }
 
@@ -211,8 +278,9 @@ public:
    bool ShuttingDown() { return shutting_down; }
    void PrepareToDie();
 
-   static const Ref<TorrentListener>& GetListener() { return listener; }
+   bool CanAccept() const;
    void Accept(int s,const sockaddr_u *a,IOBuffer *rb);
+   static bool NoTorrentCanAccept();
 
    static void SHA1(const xstring& str,xstring& buf);
    void ValidatePiece(unsigned p);
@@ -221,9 +289,11 @@ public:
 
    const TaskRefArray<TorrentPeer>& GetPeers() const { return peers; }
    void AddPeer(TorrentPeer *);
+   void CleanPeers();
 
    const xstring& GetInfoHash() const { return info_hash; }
    int GetPeersCount() const { return peers.count(); }
+   int GetConnectedPeersCount() const { return connected_peers_count; }
    int GetActivePeersCount() const { return active_peers_count; }
    int GetCompletePeersCount() const { return complete_peers_count; }
 
@@ -231,16 +301,27 @@ public:
    double GetRatio() const;
    unsigned long long TotalLength() const { return total_length; }
    unsigned PieceLength() const { return piece_length; }
-   const char *GetName() const { return name?name->get():0; }
-   const char *TrackerTimerTimeLeft() {
-      return tracker_timer.TimeLeft().toString(
-	 TimeInterval::TO_STR_TRANSLATE|TimeInterval::TO_STR_TERSE);
-   }
+   const char *GetName() const { return name?name->get():metainfo_url.get(); }
 
    void Reconfig(const char *name);
    const char *GetLogContext() { return GetName(); }
 
    void ForceValid() { force_valid=true; }
+   bool IsValidating() const { return validating; }
+
+   static int GetPort();
+   static int GetPortIPv4() { return listener?listener->GetPort():0; }
+   static int GetPortIPv6() { return listener_ipv6?listener_ipv6->GetPort():0; }
+   static const char *GetAddressIPv6() { return listener_ipv6?listener_ipv6->GetAddress():"::"; }
+   int GetWantedPeersCount() const;
+   static const xstring& GetMyPeerId() { return my_peer_id; }
+   static const xstring& GetMyKey() { return my_key; }
+
+   unsigned long long GetTotalSent() { return total_sent; }
+   unsigned long long GetTotalRecv() { return total_recv; }
+   unsigned long long GetTotalLeft() { return total_left; }
+
+   const TaskRefArray<TorrentTracker>& Trackers() { return trackers; }
 };
 
 class FDCache : public SMTask, public ResClient
@@ -257,7 +338,7 @@ class FDCache : public SMTask, public ResClient
    Timer clean_timer;
 
 public:
-   int OpenFile(const char *name,int mode);
+   int OpenFile(const char *name,int mode,off_t size=0);
    void Close(const char *name);
    int Count() const;
    void Clean();
@@ -276,6 +357,8 @@ class TorrentPeer : public SMTask, protected ProtoLog, public Networker
    Ref<Error> error;
    Torrent *parent;
 
+   int tracker_no;
+
    sockaddr_u addr;
    int sock;
    bool connected;
@@ -298,6 +381,7 @@ class TorrentPeer : public SMTask, protected ProtoLog, public Networker
    Speedometer peer_send_rate;
 
    xstring peer_id;
+   TorrentPeer *duplicate;
    bool myself;
 
    bool am_choking;
@@ -491,7 +575,7 @@ private:
 
 public:
    int Do();
-   TorrentPeer(Torrent *p,const sockaddr_u *a);
+   TorrentPeer(Torrent *p,const sockaddr_u *a,int tracker_no=-1);
    ~TorrentPeer();
    void PrepareToDie();
    void Connect(int s,IOBuffer *rb);
@@ -535,26 +619,6 @@ public:
    int Do();
 };
 
-class TorrentListener : public SMTask, protected ProtoLog, protected Networker
-{
-   Ref<Error> error;
-   int sock;
-   sockaddr_u addr;
-   xmap<Torrent*> torrents;
-   Speedometer rate;
-public:
-   TorrentListener();
-   ~TorrentListener();
-   int Do();
-   Torrent *FindTorrent(const xstring& info_hash) const { return torrents.lookup(info_hash); }
-   void AddTorrent(Torrent *);
-   void RemoveTorrent(Torrent *);
-   int GetPort() const { return addr.port(); }
-   int GetTorrentsCount() const { return torrents.count(); }
-   void Dispatch(const xstring& info_hash,int s,const sockaddr_u *remote_addr,IOBuffer *recv_buf);
-   Torrent *Lookup(const xstring& info_hash) { return torrents.lookup(info_hash); }
-};
-
 #include "Job.h"
 
 class TorrentJob : public Job
@@ -567,7 +631,7 @@ public:
    ~TorrentJob();
    int Do();
    int Done() { return done; }
-   void PrintStatus(int v,const char *tab);
+   xstring& FormatStatus(xstring&,int v,const char *tab);
    void ShowRunStatus(const SMTaskRef<StatusLine>& s);
    int AcceptSig(int);
    void PrepareToDie();
