@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -26,17 +26,16 @@
 #include "ArgV.h"
 #include "misc.h"
 
-TreatFileJob::TreatFileJob(FileAccess *s,ArgV *a) : FinderJob(s)
+TreatFileJob::TreatFileJob(FileAccess *s,ArgV *a)
+   : FinderJob(s), args(a)
 {
    quiet=false;
    failed=file_count=0;
    Need(FileInfo::NAME);
 
    curr=0;
-   first=0;
    set_maxdepth(0);
 
-   args=a;
    op=args->a0();
    Begin(a->getcurr());
 }
@@ -60,8 +59,6 @@ void TreatFileJob::Finish()
 
 TreatFileJob::~TreatFileJob()
 {
-   delete args;
-   delete first;
 }
 
 TreatFileJob::prf_res TreatFileJob::ProcessFile(const char *d,const FileInfo *fi)
@@ -93,12 +90,12 @@ TreatFileJob::prf_res TreatFileJob::ProcessFile(const char *d,const FileInfo *fi
    return res<0? PRF_ERR:PRF_OK;
 }
 
-void  TreatFileJob::PrintStatus(int v,const char *prefix)
+xstring& TreatFileJob::FormatStatus(xstring& s,int v,const char *prefix)
 {
-   SessionJob::PrintStatus(v,prefix);
+   SessionJob::FormatStatus(s,v,prefix);
    if(Done() || !curr)
-      return;
-   printf("\t`%s' [%s]\n",curr->name.get(),session->CurrentStatus());
+      return s;
+   return s.appendf("\t`%s' [%s]\n",curr->name.get(),session->CurrentStatus());
 }
 
 void  TreatFileJob::ShowRunStatus(const SMTaskRef<StatusLine>& s)
