@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -30,9 +30,9 @@ class ArgV;
 class TreatFileJob : public FinderJob
 {
 protected:
-   ArgV	 *args;
+   Ref<ArgV> args;
    const FileInfo *curr;
-   FileInfo *first;
+   Ref<FileInfo> first;
    int	 failed,file_count;
 
    virtual void	TreatCurrent(const char *d,const FileInfo *fi) = 0;
@@ -45,7 +45,7 @@ protected:
    prf_res ProcessFile(const char *d,const FileInfo *fi);
 
 public:
-   void	 PrintStatus(int,const char *);
+   xstring& FormatStatus(xstring&,int,const char *);
    void	 ShowRunStatus(const SMTaskRef<StatusLine>&);
 
    TreatFileJob(FileAccess *session,ArgV *a);
