@@ -1,11 +1,11 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-2006 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2010 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: complete.cc,v 1.76 2007/09/17 14:01:28 lav Exp $ */
+/* $Id: complete.cc,v 1.84 2011/06/16 11:09:32 lav Exp $ */
 
 #include <config.h>
 
@@ -27,7 +27,6 @@
 #include <sys/stat.h>
 #include <time.h>
 #include <unistd.h>
-#include <glob.h>
 #include <sys/time.h>
 #include <assert.h>
 #include "xmalloc.h"
@@ -47,6 +46,8 @@
 #include "misc.h"
 
 CDECL_BEGIN
+#include <glob.h>
+
 #define USE_VARARGS 1
 #define PREFER_STDARG 1
 #include "readline/readline.h"
@@ -149,9 +150,8 @@ static char *bookmark_generator(const char *text,int s)
 	    state=1;
 	 if(bookmark_prepend_bm)
 	 {
-	    char *e=alloca_strdup2("bm:",strlen(t)*3);
-	    url::encode_string(t,e+3,URL_HOST_UNSAFE);
-	    t=e;
+	    xstring& e=xstring::get_tmp("bm:");
+	    t=e.append_url_encoded(t,URL_HOST_UNSAFE);
 	 }
 	 if(strncmp(t,text,len)==0)
 	    return xstrdup(t);
@@ -356,11 +356,10 @@ static completion_type cmd_completion_type(const char *cmd,int start)
       if(alias && !TouchedAlias::IsTouched(alias,used_aliases))
       {
 	 used_aliases=new TouchedAlias(alias,used_aliases);
-	 char *cmd1=string_alloca(strlen(alias)+strlen(w)-strlen(buf)+1);
-	 strcpy(cmd1,alias);
-	 strcat(cmd1,w+strlen(buf));
-	 start=start-strlen(buf)+strlen(alias);
-	 cmd=cmd1;
+	 int buf_len=strlen(buf);
+	 const char *cmd1=xstring::cat(alias,w+buf_len,NULL);
+	 cmd=alloca_strdup(cmd1);
+	 start=start-buf_len+strlen(alias);
 	 continue;
       }
       const char *full=CmdExec::GetFullCommandName(buf);
@@ -376,7 +375,7 @@ static completion_type cmd_completion_type(const char *cmd,int start)
       if((*p=='>' || *p=='|')
       && !lftp_char_is_quoted(cmd,p-cmd))
 	 return LOCAL;
-      if(!isspace((unsigned char)*p))
+      if(!CmdExec::is_space(*p))
 	 break;
    }
 
@@ -427,24 +426,24 @@ static completion_type cmd_completion_type(const char *cmd,int start)
    int second_start=-1;
    for(int i=start; i>4; i--)
    {
-      if(!isspace(rl_line_buffer[i-1]))
+      if(!CmdExec::is_space(rl_line_buffer[i-1]))
 	 break;
-      if(!strncmp(rl_line_buffer+i-3,"-o",2) && isspace(rl_line_buffer[i-4]))
+      if(!strncmp(rl_line_buffer+i-3,"-o",2) && CmdExec::is_space(rl_line_buffer[i-4]))
       {
 	 was_o=true;
 	 break;
       }
-      if(!strncmp(rl_line_buffer+i-3,"-N",2) && isspace(rl_line_buffer[i-4]))
+      if(!strncmp(rl_line_buffer+i-3,"-N",2) && CmdExec::is_space(rl_line_buffer[i-4]))
       {
 	 was_N=true;
 	 break;
       }
-      if(i-14 >= 0 && !strncmp(rl_line_buffer+i-13, "--newer-than",12) && isspace(rl_line_buffer[i-14]))
+      if(i-14 >= 0 && !strncmp(rl_line_buffer+i-13, "--newer-than",12) && CmdExec::is_space(rl_line_buffer[i-14]))
       {
 	 was_N=true;
 	 break;
       }
-      if(!strncmp(rl_line_buffer+i-3,"-O",2) && isspace(rl_line_buffer[i-4]))
+      if(!strncmp(rl_line_buffer+i-3,"-O",2) && CmdExec::is_space(rl_line_buffer[i-4]))
       {
 	 was_O=true;
 	 break;
@@ -503,7 +502,7 @@ static completion_type cmd_completion_type(const char *cmd,int start)
 	 return LOCAL;
       if(have_N)
 	 arg--;
-      if(have_R ^ arg!=1)
+      if(have_R ^ (arg!=1))
 	 return LOCAL_DIR;
       if(arg>2)
 	 return NO_COMPLETION;
@@ -666,7 +665,7 @@ static char **lftp_completion (const char *text,int start,int end)
       bool tilde_expanded=false;
       const char *home=getenv("HOME");
       int home_len=xstrlen(home);
-      pat=(char*)alloca((len+home_len)*2+10);
+      pat=string_alloca((len+home_len)*2+10);
       if(len>0 && home_len>0 && text[0]=='~' && (len==1 || text[1]=='/'))
       {
 	 glob_quote(pat,home,home_len);
@@ -742,7 +741,7 @@ static char **lftp_completion (const char *text,int start,int end)
 	 break;
       }
 
-      pat=(char*)alloca(len*2+10);
+      pat=string_alloca(len*2+10);
       glob_quote(pat,text,len);
 
       if(pat[0]=='~' && pat[1]==0)
@@ -767,11 +766,11 @@ static char **lftp_completion (const char *text,int start,int end)
 
       if(rg)
       {
-	 rg->glob->NoMatchPeriod();
-	 rg->glob->NoInhibitTilde();
+	 rg->NoMatchPeriod();
+	 rg->NoInhibitTilde();
 	 if(fso.patterns_casefold) {
 	    rl_variable_bind("completion-ignore-case", "1");
-	    rg->glob->CaseFold();
+	    rg->CaseFold();
 	 } else
 	    rl_variable_bind("completion-ignore-case", "0");
 
@@ -1237,13 +1236,12 @@ static int lftp_complete_remote(int count,int key)
 
 int lftp_rl_getc(FILE *file)
 {
-   int res;
+   SignalHook::DoCount(SIGINT);
    CharReader r(fileno(file));
-
    for(;;)
    {
       SMTask::Schedule();
-      res=r.GetChar();
+      int res=r.GetChar();
       if(res==r.EOFCHAR)
 	 return EOF;
       if(res!=r.NOCHAR)
