@@ -18,23 +18,23 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: lftp.cc,v 1.84 2009/11/17 08:18:37 lav Exp $ */
+/* $Id: lftp.cc,v 1.89 2011/06/17 11:05:44 lav Exp $ */
 
 #include <config.h>
 
 #include "modconfig.h"
 
 #include "trio.h"
+#include <sys/types.h>
+#include <sys/stat.h> // for mkdir()
 #include <stdlib.h>
 #include <errno.h>
 #include <unistd.h>
-#include <sys/types.h>
-#include <sys/stat.h> // for mkdir()
-#include "xstring.h"
 #include <fcntl.h>
 #include <locale.h>
 #include <ctype.h>
 
+#include "xstring.h"
 #include "xmalloc.h"
 #include "alias.h"
 #include "CmdExec.h"
@@ -50,16 +50,19 @@
 #include "ConnectionSlot.h"
 #include "misc.h"
 #include "ArgV.h"
+#include "attach.h"
 
 #include "configmake.h"
 
 #include "lftp_rl.h"
 #include "complete.h"
 
+CDECL_BEGIN
+#include <glob.h>
+CDECL_END
 
 #define top_exec CmdExec::top
 
-
 void  hook_signals()
 {
    SignalHook::DoCount(SIGHUP);
@@ -303,16 +306,51 @@ CMD(history)
 
    return 0;
 }
+CMD(attach)
+{
+   const char *pid_s=args->getarg(1);
+   if(!pid_s) {
+      xstring& path=AcceptTermFD::get_sock_path(1);
+      path.rtrim('1');
+      path.append('*');
+      glob_t g;
+      glob(path, 0, NULL, &g);
+      if(g.gl_pathc==0) {
+      not_found:
+	 eprintf("%s: no backgrounded lftp processes found.\n",args->a0());
+	 globfree(&g);
+	 return 0;
+      }
+      pid_s=strrchr(g.gl_pathv[0],'-');
+      if(pid_s)
+	 pid_s=alloca_strdup(pid_s+1);
+      globfree(&g);
+      if(!pid_s)
+	 goto not_found;
+   }
+   int pid=atoi(pid_s);
+   SMTaskRef<SendTermFD> term_sender(new SendTermFD(pid));
+   while(!term_sender->Done()) {
+      SMTask::Schedule();
+      SMTask::Block();
+   }
+   exit_code=0;
+   if(term_sender->Failed()) {
+      eprintf("%s\n",term_sender->ErrorText());
+      exit_code=1;
+   }
+   return 0;
+}
 #undef args
 #undef exit_code
 #undef output
 #undef session
 #undef eprintf
 
+
 static void sig_term(int sig)
 {
-   time_t t=time(0);
-   printf(_("[%lu] Terminated by signal %d. %s"),(unsigned long)getpid(),sig,ctime(&t));
+   printf(_("[%u] Terminated by signal %d. %s\n"),(unsigned)getpid(),sig,SMTask::now.IsoDateTime());
    if(top_exec) {
       top_exec->KillAll();
       alarm(30);
@@ -324,7 +362,45 @@ static void sig_term(int sig)
    exit(1);
 }
 
-static void move_to_background()
+static void detach()
+{
+   SignalHook::Ignore(SIGINT);
+   SignalHook::Ignore(SIGHUP);
+   SignalHook::Ignore(SIGTSTP);
+
+   const char *home=get_lftp_home();
+   if(home)
+   {
+      xstring& log=xstring::get_tmp(home);
+      if(access(log,F_OK)==-1)
+	 log.append("_log");
+      else
+	 log.append("/log");
+
+      int fd=open(log,O_WRONLY|O_APPEND|O_CREAT,0600);
+      if(fd>=0)
+      {
+	 dup2(fd,1); // stdout
+	 dup2(fd,2); // stderr
+	 if(fd!=1 && fd!=2)
+	    close(fd);
+      }
+      Log::global->ShowPID();
+      Log::global->ShowTime();
+      Log::global->ShowContext();
+   }
+   close(0);	  // close stdin.
+   open("/dev/null",O_RDONLY); // reopen it, just in case.
+
+#ifdef HAVE_SETSID
+   setsid();	  // start a new session.
+#endif
+
+   SignalHook::Handle(SIGTERM,sig_term);
+}
+
+SMTaskRef<AcceptTermFD> term_acceptor;
+static int move_to_background()
 {
    // notify jobs
    Job::lftpMovesToBackground_ToAll();
@@ -332,73 +408,47 @@ static void move_to_background()
    SMTask::RollAll(TimeInterval(1,0));
    // if all jobs terminated, don't really move to bg.
    if(Job::NumberOfJobs()==0)
-      return;
+      return 0;
 
    fflush(stdout);
    fflush(stderr);
 
-   pid_t pid=fork();
+   static bool detached;
+   pid_t pid=0;
+   if(!detached)
+      pid=fork();
    switch(pid)
    {
    case(0): // child
    {
-      SignalHook::Ignore(SIGINT);
-      SignalHook::Ignore(SIGQUIT);
-      SignalHook::Ignore(SIGHUP);
-      SignalHook::Ignore(SIGTSTP);
-
-      const char *home=get_lftp_home();
-      if(home)
-      {
-	 char *log=(char*)alloca(strlen(home)+1+3+1);
-	 sprintf(log,"%s",home);
-	 if(access(log,F_OK)==-1)
-	    strcat(log,"_log");
-	 else
-	    strcat(log,"/log");
-
-	 int fd=open(log,O_WRONLY|O_APPEND|O_CREAT,0600);
-	 if(fd>=0)
-	 {
-	    dup2(fd,1); // stdout
-	    dup2(fd,2); // stderr
-	    if(fd!=1 && fd!=2)
-	       close(fd);
-	 }
-	 Log::global->ShowPID();
-	 Log::global->ShowTime();
-	 Log::global->ShowContext();
+      if(!detached) {
+	 detach();
+	 detached=true;
+	 printf(_("[%u] Started.  %s\n"),(unsigned)getpid(),SMTask::now.IsoDateTime());
       }
-      close(0);	  // close stdin.
-      open("/dev/null",O_RDONLY); // reopen it, just in case.
-
-#ifdef HAVE_SETSID
-      setsid();	  // start a new session.
-#endif
-
-      pid=getpid();
-      time_t t=time(0);
-      SignalHook::Handle(SIGTERM,sig_term);
-      printf(_("[%lu] Started.  %s"),(unsigned long)pid,ctime(&t));
+      if(!term_acceptor)
+	 term_acceptor=new AcceptTermFD();
       for(;;)
       {
 	 SMTask::Schedule();
 	 if(Job::NumberOfJobs()==0)
 	    break;
 	 SMTask::Block();
+	 if(term_acceptor->Accepted())
+	    return 1;
       }
-      t=time(0);
-      printf(_("[%lu] Finished. %s"),(unsigned long)pid,ctime(&t));
-      return;
+      printf(_("[%u] Finished. %s\n"),(unsigned)getpid(),SMTask::now.IsoDateTime());
+      return 0;
    }
    default: // parent
-      printf(_("[%lu] Moving to background to complete transfers...\n"),
-	       (unsigned long)pid);
+      printf(_("[%u] Moving to background to complete transfers...\n"),
+	       (unsigned)pid);
       fflush(stdout);
       _exit(0);
    case(-1):
       perror("fork()");
    }
+   return 0;
 }
 
 int lftp_slot(int count,int key)
@@ -452,6 +502,8 @@ int   main(int argc,char **argv)
 	 " -l  List the history (default).\n"
 	 "Optional argument cnt specifies the number of history lines to list,\n"
 	 "or \"all\" to list all entries.\n"));
+   CmdExec::RegisterCommand("attach",cmd_attach,"attach [PID]",
+      N_("Attach the terminal to specified backgrounded lftp process.\n"));
 
    top_exec=new CmdExec(0,0);
    hook_signals();
@@ -476,6 +528,7 @@ int   main(int argc,char **argv)
    lftp_feeder=new ReadlineFeeder(args);
 
    top_exec->ExecParsed(args.borrow());
+revived:
    top_exec->WaitDone();
    int exit_code=top_exec->ExitCode();
 
@@ -485,11 +538,30 @@ int   main(int argc,char **argv)
    if(Job::NumberOfJobs()>0)
    {
       top_exec->SetInteractive(false);
-      move_to_background();
+      if(term_acceptor) {
+	 printf(_("[%u] Detaching from the terminal to complete transfers...\n"),
+	       (unsigned)getpid());
+	 fflush(stdout);
+	 term_acceptor->Detach();
+	 detach();
+	 printf(_("[%u] Detached from terminal. %s\n"),(unsigned)getpid(),SMTask::now.IsoDateTime());
+      }
+      if(move_to_background())
+      {
+	 top_exec->SetInteractive(true);
+	 top_exec->SetStatusLine(new StatusLine(1));
+	 top_exec->SetCmdFeeder(new ReadlineFeeder(args));
+	 goto revived;
+      }
    }
    top_exec->KillAll();
    top_exec=0;
 
+   if(term_acceptor) {
+      printf(_("[%u] Exiting and detaching from the terminal.\n"),(unsigned)getpid());
+      fflush(stdout);
+   }
+
    Job::Cleanup();
    ConnectionSlot::Cleanup();
    SessionPool::ClearAll();
@@ -499,6 +571,5 @@ int   main(int argc,char **argv)
    SignalHook::Cleanup();
    Log::Cleanup();
    SMTask::Cleanup();
-
    return exit_code;
 }
