@@ -1,11 +1,11 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-2007 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2010 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -28,17 +28,27 @@
 
 class mvJob : public SessionJob
 {
-   int	 failed;
+   xstring_c from;
+   xstring to;
+   FA::open_mode m;
+   bool	 remove_target;
+   bool	 failed;
+   bool	 done;
+
 public:
    int	 Do();
-   int	 Done() { return session->IsClosed(); }
+   int	 Done() { return done; }
    int	 ExitCode() { return failed; }
 
-   void	 PrintStatus(int,const char *);
+   xstring& FormatStatus(xstring&,int,const char *);
    void	 ShowRunStatus(const SMTaskRef<StatusLine>&);
    void	 SayFinal();
 
-   mvJob(FileAccess *session,const char *from,const char *to);
+   void	 doOpen() const;
+   const char *cmd() const { return m==FA::RENAME ? "mv" : "ln"; }
+
+   mvJob(FileAccess *session,const char *f,const char *t,FA::open_mode m=FA::RENAME);
+   void RemoveTargetFirst() { remove_target=true; }
 };
 
 #endif // MVJOB_H
