@@ -1,11 +1,11 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-2004 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2007 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,14 +18,14 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: pgetJob.h,v 1.13 2006/07/03 13:23:56 lav Exp $ */
+/* $Id: pgetJob.h,v 1.21 2011/04/29 04:58:29 lav Exp $ */
 
 #ifndef PGETJOB_H
 #define PGETJOB_H
 
-#include "GetJob.h"
+#include "CopyJob.h"
 
-class pgetJob : public GetJob
+class pgetJob : public CopyJob
 {
    class ChunkXfer : public CopyJob
    {
@@ -34,13 +34,12 @@ class pgetJob : public GetJob
       off_t start;
       off_t limit;
 
-      ChunkXfer(FileCopy *c,const char *remote,off_t start,off_t limit);
-      ~ChunkXfer();
+      ChunkXfer(FileCopy *c,const char *n,off_t start,off_t limit);
    };
 
-   ChunkXfer   **chunks;
+   TaskRefArray<ChunkXfer> chunks;
    int	 max_chunks;
-   int	 num_of_chunks;
+   off_t chunks_bytes;
    void InitChunks(off_t offset,off_t size);
 
    off_t start0;
@@ -54,12 +53,12 @@ class pgetJob : public GetJob
    bool pget_cont:1;
 
    void free_chunks();
-   ChunkXfer *NewChunk(const char *remote,FDStream *local,off_t start,off_t limit);
+   ChunkXfer *NewChunk(const char *remote,off_t start,off_t limit);
 
    long total_eta;
 
    Timer status_timer;
-   char *status_file;
+   xstring status_file;
    void SaveStatus();
    void LoadStatus();
    void LoadStatus0();
@@ -69,15 +68,27 @@ protected:
 
 public:
    int Do();
-   void ShowRunStatus(StatusLine *s);
-   void PrintStatus(int,const char *);
-   void ListJobs(int verbose,int indent=0);
+   void ShowRunStatus(const SMTaskRef<StatusLine>&);
+   xstring& FormatStatus(xstring&,int,const char *);
+   xstring& FormatJobs(xstring&,int verbose,int indent);
 
-   pgetJob(FileAccess *s,ArgV *args,bool cont);
+   pgetJob(FileCopy *c1,const char *n,int m=0);
    ~pgetJob();
+   void PrepareToDie();
 
    void SetMaxConn(int n) { max_chunks=n; }
 
+   off_t GetBytesCount() { return chunks_bytes+c->GetBytesCount(); }
+};
+
+class pCopyJobCreator : public CopyJobCreator
+{
+public:
+   int max_chunks;
+   pCopyJobCreator(int n) : max_chunks(n) {}
+   CopyJob *New(FileCopy *c,const char *n,const char *o) const {
+      return new pgetJob(c,n,max_chunks);
+   }
 };
 
 #endif//PGETJOB_H
