@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 2007 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 2007-2010 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,13 +18,14 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: xstring.cc,v 1.25 2009/07/22 08:31:00 lav Exp $ */
+/* $Id: xstring.cc,v 1.33 2011/05/10 05:52:34 lav Exp $ */
 
 #include <config.h>
 #include <string.h>
 #include <mbswidth.h>
 #include "xstring.h"
 #include "trio.h"
+#include "c-ctype.h"
 
 int xstrcmp(const char *s1,const char *s2)
 {
@@ -57,28 +58,36 @@ size_t xstrlen(const char *s)
    return strlen(s);
 }
 
-void xstring::get_space(size_t s,size_t g)
+void xstring::get_space(size_t s)
+{
+   get_space2(s,32);
+}
+void xstring::get_space2(size_t s,size_t g)
 {
    if(!buf)
       buf=(char*)xmalloc(size=s+1);
    else if(size<s+1)
-      buf=(char*)realloc(buf,size=(s|(g-1))+1);
+      buf=(char*)xrealloc(buf,size=(s|(g-1))+1);
    else if(size>=g*8 && s+1<=size/2)
-      buf=(char*)realloc(buf,size/=2);
+      buf=(char*)xrealloc(buf,size/=2);
    buf[s]=0;
 }
+char *xstring::add_space(size_t s)
+{
+   if(size<=len+s)
+      get_space(len+s);
+   return get_non_const()+len;
+}
 
 void xstring::init(const char *s,int len)
 {
    init();
-   if(s)
-      nset(s,len);
+   nset(s,len);
 }
 void xstring::init(const char *s)
 {
    init();
-   if(s)
-      set(s);
+   set(s);
 }
 
 xstring& xstring::nset(const char *s,int len)
@@ -309,6 +318,12 @@ xstring& xstring::get_tmp()
    static int i;
    return revolver[i=(i+1)&15];
 }
+char *xstring::tmp_buf(int n)
+{
+   xstring& buf=get_tmp();
+   buf.get_space(n-1);	// get_space adds a tail byte again
+   return buf.get_non_const();
+}
 xstring& xstring::format(const char *fmt, ...)
 {
    va_list va;
@@ -379,7 +394,7 @@ const char *xstring::dump_to(xstring& buf) const
 	    buf.appendf("%02X",(unsigned char)*s++);
 	 buf.append('>');
       } else {
-	 buf.appendf("<long binary, %d bytes>",length());
+	 buf.appendf("<long binary, %d bytes>",(int)length());
       }
    } else {
       while(len>0) {
@@ -406,4 +421,65 @@ const char *xstring::dump_to(xstring& buf) const
    return buf;
 }
 
+int xstring0::_url_decode(size_t len)
+{
+   if(!buf)
+      return 0;
+   char *store=(char*)memchr(buf,'%',len);
+   if(!store)
+      return len;
+   const char *p=store;
+   int rest=len-(p-buf);
+   while(rest>=3)
+   {
+      if(*p=='%' && c_isxdigit(p[1]) && c_isxdigit(p[2]))
+      {
+	 int n;
+	 if(sscanf(p+1,"%2x",&n)==1)
+	 {
+	    *store++=n;
+	    p+=3;
+	    rest-=3;
+	    continue;
+	 }
+      }
+      *store++=*p++;
+      rest--;
+   }
+   while(rest>0) {
+      *store++=*p++;
+      rest--;
+   }
+   return store-buf;
+}
+
+xstring& xstring::url_decode()
+{
+   set_length(_url_decode(length()));
+   return *this;
+}
+xstring_c& xstring_c::url_decode()
+{
+   set_length(_url_decode(length()));
+   return *this;
+}
+
+/* Encode the unsafe characters in a given string, producing %XX encoded string. */
+#define need_quote(c) (c_iscntrl((c)) || !c_isascii((c)) || strchr(unsafe,(c)))
+xstring& xstring::append_url_encoded(const char *s,int len,const char *unsafe)
+{
+   if(!s)
+      return *this;
+   add_space(len+len/4);
+   while(len-->0)
+   {
+      char c=*s++;
+      if (need_quote(c))
+	 appendf("%%%02X",(unsigned char)c);
+      else
+	 append(c);
+   }
+   return *this;
+}
+
 xstring xstring::null;
